--[[
Interface: 1.5.1.0 b1580

Copyright (C) GtX (Andy), 2018

Author: GtX | Andy
Date: 14.04.2018
Version: 1.0.0.0

Support, Guide and Example Mod: http://ls-modcompany.com

History:
V 1.0.0.0 	@ 	14.04.2018 	- 	Project Started

*****************************************************************************************************************
** Thankyou **
**************

Sven777b @ http://ls-landtechnik.com	-	Allowing me to use parts of his code as found in ‘Beleuchtung v3.1.1’.

Inerti 		-	'extraLight' ideas and script suggestions, Single and Multiplayer testing.

Nicolina 	-	Single and Multiplayer testing.

******************************************************************************************************************
** Important **
***************

frei verwendbar - keine erlaubnis nötig    /    Modifikationen erst nach Rücksprache!

free for use - no permission needed.       /    Modifications and Conversions only with my permission!

*******************************************************************************************************************
]]

LightExtension = {}

function LightExtension.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Lights, specializations)
end

function LightExtension:load(savegame)
	self.setBeaconSound = LightExtension.setBeaconSound
	self.setUserLightState = LightExtension.setUserLightState
	self.setExtraLightState = LightExtension.setExtraLightState
	self.setStrobeState = LightExtension.setStrobeState
	self.setBeaconLightsVisibility  = Utils.appendedFunction(self.setBeaconLightsVisibility, LightExtension.setBeaconLightsVisibility)
	self.setLightsTypesMask  = Utils.appendedFunction(self.setLightsTypesMask, LightExtension.setLightsTypesMask)

	self.lightExt = {}

	if hasXMLProperty(self.xmlFile, "vehicle.lightExtension.strobeLights") then
		self.lightExt.strobes = {}
		self.strobesActive = false
		self.beaconSync = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.lightExtension.strobeLights#beaconSync"), false)
		self.strobeInputHelp = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.lightExtension.strobeLights#inputHelpOnlyIfSelected"), true)
		self.strobeInputSound = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.lightExtension.strobeLights#playInputSound"), true)
		local realStrobeLights = g_gameSettings:getValue("realBeaconLights")
		local highProfile = self:getUseHighProfile()

		local i = 0
		while true do
			local key = string.format("vehicle.lightExtension.strobeLights.strobe(%d)", i)

			if not hasXMLProperty(self.xmlFile, key) then
				break
			end

			local decoration = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key.."#decoration"))
			if decoration ~= nil then
				local realLight = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key.."#realLight"))
				local seq = getXMLString(self.xmlFile, key.."#sequence")
				local strobe = {}
				strobe.ls = false
				strobe.ns = 0
				strobe.t = 1
				strobe.a = false
				if seq ~= nil then
					strobe.rnd = false
					strobe.sq = {Utils.getVectorFromString(seq)}
					strobe.inv = Utils.getNoNil(getXMLBool(self.xmlFile, key.."#invert"), false)
					strobe.ls = strobe.inv
					strobe.ss = 1
				else
					strobe.rnd = true
					strobe.rndnn = Utils.getNoNil(getXMLInt(self.xmlFile, key.."#minOn"),100)
					strobe.rndxn = Utils.getNoNil(getXMLInt(self.xmlFile, key.."#maxOn"),100)
					strobe.rndnf = Utils.getNoNil(getXMLInt(self.xmlFile, key.."#minOff"),100)
					strobe.rndxf = Utils.getNoNil(getXMLInt(self.xmlFile, key.."#maxOff"),400)
					math.randomseed(getTime()) math.random()
				end
				strobe.decoration = decoration
				setVisibility(decoration, false)

				if realLight ~= nil then
					if realStrobeLights and highProfile then
						strobe.realLight = realLight
						-- local defaultColor = {getLightColor(realLight)}
					end
					setVisibility(realLight, false)
				end

				table.insert(self.lightExt.strobes, strobe)
			else
				print("Error: [ LightExtension.lua ] No decoration light set at ("..key..") in "..self.configFileName.." This is a minimum requirement. Please set.")
			end
			i = i + 1
		end
	end

	if self.isSteerable then
		if hasXMLProperty(self.xmlFile, "vehicle.lightExtension.extraLight") then
			self.lightExt.extra = {}
			local extra = {}

			local lightControlActive = false
			extra.inputOnly = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.lightExtension.extraLight#inputOnly"), true)
			self.extraInputHelp = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.lightExtension.extraLight#inputHelpOnlyIfSelected"), true)
			self.extraInputSound = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.lightExtension.extraLight#playInputSound"), true)

			local usingDesign = false
			local usingVehicle = false
			self.extraLightEnabled = false
			if self.configurations["design"] ~= nil then
				local designConfigs = {Utils.getVectorFromString(getXMLString(self.xmlFile, "vehicle.lightExtension.extraLight#showWithDesignConfigurations"))}
				if #designConfigs > 0 then
					usingDesign = true
					for _,id in pairs (designConfigs) do
						if id == self.configurations["design"] then
							self.extraLightEnabled = true
							break
						end
					end
				end
			end

			if self.configurations["vehicleType"] ~= nil then
				local vehicleConfigs = {Utils.getVectorFromString(getXMLString(self.xmlFile, "vehicle.lightExtension.extraLight#showWithVehicleTypeConfigurations"))}
				if #vehicleConfigs > 0 then
					if not usingDesign then
						usingVehicle = true
						for _,id in pairs (vehicleConfigs) do
							if id == self.configurations["vehicleType"] then
								self.extraLightEnabled = true
								break
							end
						end
					else
						print("Error: [ LightExtension.lua ] 'vehicle.lightExtension.extraLight#showWithDesignConfigurations' already in use. Please remove 'vehicle.lightExtension.extraLight#showWithVehicleTypeConfigurations' in "..self.configFileName)
					end
				end
			end

			if not usingDesign and not usingVehicle then
				self.extraLightEnabled = true
			end

			if not extra.inputOnly then
				extra.syncLightType = getXMLInt(self.xmlFile, "vehicle.lightExtension.extraLight#syncLightType")
				if extra.syncLightType ~= nil then
					if extra.syncLightType <= self.numLightTypes then
						lightControlActive = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.lightExtension.extraLight#defaultOn"), true)

						if savegame ~= nil then
							lightControlActive = Utils.getNoNil(getXMLBool(savegame.xmlFile, savegame.key.."#extraLightActive"), lightControlActive)
							self:setUserLightState(lightControlActive, true)
						end
					else
						print("Warning [LightExtension.lua]: lightType '"..extra.syncLightType.."' does not exist in '"..self.configFileName.."' Set syncLightType from '0 to "..self.numLightTypes"' only!")
					end
				else
					print("Warning [LightExtension.lua]: 'vehicle.lightExtension.extraLight#syncLightType' was not found and 'inputOnly = false' in '"..self.configFileName.."'!  'inputOnly' has been activated please fix this.")
					extra.inputOnly = true
				end
			end

			self.lightControlActive = lightControlActive
			self.autoLightState = false
			extra.deco = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.lightExtension.extraLight#decoration"))
			extra.realLow = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.lightExtension.extraLight#realLightLow"))
			extra.realHigh = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.lightExtension.extraLight#realLightHigh"))
			extra.lightState = false
			if extra.deco ~= nil then
				setVisibility(extra.deco, false)

				local hasLow = extra.realLow ~= nil
				local hasHigh = extra.realHigh ~= nil

				if hasLow and not hasHigh then
					print("Error: (LightExtension.lua) high-profile light index missing in '"..self.configFileName.."'")
				elseif not hasLow and hasHigh then
					print("Error: (LightExtension.lua) low-profile light index missing in '"..self.configFileName.."'")
				end

				if hasLow and hasHigh then
					setVisibility(extra.realLow, false)
					setVisibility(extra.realHigh, false)
				end

				self.lightExt.extra = extra
			else
				print("Error: (LightExtension.lua) 'decoration' light missing in '"..self.configFileName.."'")
			end
		end

		if hasXMLProperty(self.xmlFile, "vehicle.lightExtension.beaconSound") then
			self.lightExt.beaconSound = {}
			local sample = SoundUtil.loadSample(self.xmlFile, {}, "vehicle.lightExtension.beaconSound", nil, self.baseDirectory, self.components[1].node)
			if sample ~= nil then
				self.lightExt.beaconSound.sample = sample
				if self.lightExt.strobes ~= nil then
					self.lightExt.beaconSound.onStrobe = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.lightExtension.beaconSound#strobeOnly"), true)
				else
					self.lightExt.beaconSound.onStrobe = false
				end
				self.lightExt.beaconSound.active = false
			else
				print("[ LightExtension.lua ] No sound file set at (vehicle.lightExtension.beaconSound) This is a minimum requirement. Please set.")
			end
		end

		local percent = getXMLFloat(self.xmlFile, "vehicle.lightExtension.combineFillWarning#percent")
		if percent ~= nil then
			if SpecializationUtil.hasSpecialization(Combine, self.specializations) and self.fillUnits[self.overloading.fillUnitIndex].fillLevel ~= nil then
				self.lightExt.cfw = {}
				self.lightExt.cfw.p = percent
				-- self.lightExt.cfw.pa = false
				self.lightExt.cfw.ba = false
				self.lightExt.cfw.a = false
				self.lightExt.cfw.turnOnBeacons = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.lightExtension.combineFillWarning#turnOnBeacons"), false)
				-- self.lightExt.cfw.forceOpenPipe = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.lightExtension.combineFillWarning#forceOpenPipe"), false)
				-- self.lightExt.cfw.forceClosePipe = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.lightExtension.combineFillWarning#forceClosePipe"), false)
				local soundFile = getXMLString(self.xmlFile, "vehicle.lightExtension.combineFillWarning.inCabAlarm#file")
				if soundFile ~= nil then
					self.lightExt.cfw.inCabAlarm = SoundUtil.loadSample(self.xmlFile, {}, "vehicle.lightExtension.combineFillWarning.inCabAlarm", nil, self.baseDirectory)
					self.lightExt.cfw.hasSound = true
				else
					self.lightExt.cfw.hasSound = false
				end
				local seconds = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.lightExtension.combineFillWarning.inCabAlarm#alarmDuration"), 3)
				self.lightExt.cfw.seconds = seconds * 1000
				self.lightExt.cfw.counter = self.lightExt.cfw.seconds
			else
				print("[ LightExtension.lua ] Error "..self.configFileName.." 'combineFillWarning' is only for use on vehicles with 'Combine Specialization' ")
			end
		end
	end
end

function LightExtension:getSaveAttributesAndNodes(nodeIdent)
    if self.lightExt.extra ~= nil and not self.lightExt.extra.inputOnly then
        local attributes = 'extraLightActive="'..tostring(self.lightControlActive)..'"'
        return attributes, nil
    end
    return nil
end

function LightExtension:delete()
	if self.isClient then
		if self.lightExt.beaconSound then
			SoundUtil.deleteSample(self.lightExt.beaconSound.sample)
		end
		if self.lightExt.cfw then
			if self.lightExt.cfw.inCabAlarm then
				SoundUtil.deleteSample(self.lightExt.cfw.inCabAlarm)
			end
		end
    end
end

function LightExtension:readStream(streamId, connection)
	if self.lightExt.strobes ~= nil then
		self.strobesActive = streamReadBool(streamId)
	end
	if self.lightExt.extra ~= nil then
		if not self.lightExt.extra.inputOnly then
			self.lightControlActive = streamReadBool(streamId)
		end
	end
end

function LightExtension:writeStream(streamId, connection)
	if self.lightExt.strobes ~= nil then
		streamWriteBool(streamId, self.strobesActive)
	end
	if self.lightExt.extra then
		if not self.lightExt.extra.inputOnly then
			streamWriteBool(streamId, self.lightControlActive)
		end
	end
end

function LightExtension:update(dt)
	if self.isClient and self:getIsActiveForLights() then
		if self:getIsActiveForInput(false, false) and self.isSteerable then

			-- Light Control Input
			if self.lightExt.extra ~= nil then
				if self.extraLightEnabled then
					if self.lightExt.extra.inputOnly then
						if InputBinding.hasEvent(InputBinding.LE_EXTRALIGHTS) then
							self:setUserLightState(not self.lightControlActive)
							if self.extraInputSound then
								SoundUtil.playSample(g_currentMission.sampleToggleLights, 1, 0, 1)
							end
						end
					else
						if bitAND(self.lightsTypesMask, 2^self.lightExt.extra.syncLightType) ~= 0 then
							if InputBinding.hasEvent(InputBinding.LE_EXTRALIGHTS) then
								self:setUserLightState(not self.lightControlActive)
								if self.extraInputSound then
									SoundUtil.playSample(g_currentMission.sampleToggleLights, 1, 0, 1)
								end
							end
						end
					end
				end
			end
			-- Strobes Input
			if self.lightExt.strobes ~= nil then
				if not self.beaconSync then
					if InputBinding.hasEvent(InputBinding.LE_STROBELIGHTS) then
						self:setStrobeState(not self.strobesActive)
						if self.strobeInputSound then
							SoundUtil.playSample(g_currentMission.sampleToggleLights, 1, 0, 1)
						end
					end
				end
			end
		end
	end

	-- Strobe Main
	if self.lightExt.strobes ~= nil then
		if self.strobesActive then
			self.resetStrobes = true
			for _,st in ipairs(self.lightExt.strobes) do
				if st.t > st.ns then
					st.ls = not st.ls
					setVisibility(st.decoration, st.ls)
					if st.realLight ~= nil then
						setVisibility(st.realLight, st.ls)
					end
					st.t = 0;
					if st.rnd then
						if st.ls then
							st.ns = math.random(st.rndnn,st.rndxn)
						else
							st.ns = math.random(st.rndnf,st.rndxf)
						end
					else
						st.ss = st.ss + 1
						if st.ss > table.getn(st.sq) then st.ss = 1; end
						st.ns = st.sq[st.ss]
					end
				else
					st.t = st.t + dt
				end
			end
		else
			if self.resetStrobes then
				for _,st in ipairs(self.lightExt.strobes) do
					setVisibility(st.decoration, false)
					if st.realLight ~= nil then
						setVisibility(st.realLight, false)
					end
				end
				self.resetStrobes = false
			end
		end
	end
end

function LightExtension:updateTick(dt)
	-- Combine Warning
	if self.isSteerable then
		if self:getIsActive() and not self.isHired then
			if self.lightExt.cfw ~= nil then
				local fillLevel = self:getUnitFillLevel(self.overloading.fillUnitIndex)
				local capacity = self:getUnitCapacity(self.overloading.fillUnitIndex)
				local percent = fillLevel * 100 / capacity
				if not self.lightExt.cfw.a then
					if percent >= self.lightExt.cfw.p then
						if not self.beaconLightsActive and self.lightExt.cfw.turnOnBeacons then
							self:setBeaconLightsVisibility(true)
							self.lightExt.cfw.ba = true
						end
						self.lightExt.cfw.counter = 0
						self.lightExt.cfw.a = true
					end
				else
					if percent < self.lightExt.cfw.p then
						if self.lightExt.cfw.ba and self.lightExt.cfw.turnOnBeacons then
							self:setBeaconLightsVisibility(false)
							self.lightExt.cfw.ba = false
						end
						self.lightExt.cfw.counter = self.lightExt.cfw.seconds
						self.lightExt.cfw.a = false
					end
				end
				-- if self.hasPipe and self.lightExt.cfw.forceOpenPipe then
					-- if not self.lightExt.cfw.pa then
						-- if percent >= self.lightExt.cfw.p then
							-- if self.pipeTargetState ~= 2 then
								-- self:setPipeState(2)
							-- end
							-- self.lightExt.cfw.pa = true
						-- end
					-- else
						-- if self.lightExt.cfw.forceClosePipe then
							-- if fillLevel <= 0 and not self.combineIsFilling then
								-- if self.pipeTargetState ~= 1 then
									-- self:setPipeState(1)
								-- end
								-- self.lightExt.cfw.pa = false
							-- end
						-- else
							-- if percent < self.lightExt.cfw.p then
								-- self.lightExt.cfw.pa = false
							-- end
						-- end
					-- end
				-- end
			end
		end

		if self.lightExt.cfw ~= nil and self.lightExt.cfw.hasSound and not self.isHired then
			if self.lightExt.cfw.counter < self.lightExt.cfw.seconds then
				self.lightExt.cfw.counter = self.lightExt.cfw.counter + dt
				if self:getIsActiveForSound() then
					SoundUtil.playSample(self.lightExt.cfw.inCabAlarm, 0, 0, nil)
				end
			else
				if SoundUtil.isSamplePlaying(self.lightExt.cfw.inCabAlarm) then
					SoundUtil.stopSample(self.lightExt.cfw.inCabAlarm)
				end
			end
		end
	end
end

function LightExtension:draw()
	if self.isClient and self:getIsActiveForLights() then
		if self.lightExt.strobes ~= nil and self.isSteerable then
			if not self.beaconSync then
				if self:getIsActiveForInput(self.strobeInputHelp, false) then
					if self.strobesActive then
						g_currentMission:addHelpButtonText(g_i18n:getText("action_LE_StrobesOff"), InputBinding.LE_STROBELIGHTS)
					else
						g_currentMission:addHelpButtonText(g_i18n:getText("action_LE_StrobesOn"), InputBinding.LE_STROBELIGHTS)
					end
				end
			end
		end

		if self.lightExt.extra ~= nil then
			if self.extraLightEnabled then
				if self:getIsActiveForInput(self.extraInputHelp, false) then
					local active = false
					if self.lightExt.extra.inputOnly then
						active = true
					else
						active = bitAND(self.lightsTypesMask, 2^self.lightExt.extra.syncLightType) ~= 0
					end
					if active then
						if self.lightControlActive then
							g_currentMission:addHelpButtonText(g_i18n:getText("action_LE_ExtraOff"), InputBinding.LE_EXTRALIGHTS)
						else
							g_currentMission:addHelpButtonText(g_i18n:getText("action_LE_ExtraOn"), InputBinding.LE_EXTRALIGHTS)
						end
					end
				end
			end
		end
	end
end

function LightExtension:onDeactivateSounds()
    if self.isClient then
		-- force stop the alarm is the player leaves the vehicle.
		if self.lightExt.cfw ~= nil and self.lightExt.cfw.hasSound then
			SoundUtil.stopSample(self.lightExt.cfw.inCabAlarm, true)
		end
    end
end

function LightExtension:setBeaconSound(playSound)
	if self.isClient then
		local sample = self.lightExt.beaconSound.sample
		if playSound then
			SoundUtil.play3DSample(sample)
		else
			SoundUtil.stop3DSample(sample)
		end
		self.lightExt.beaconSound.active = playSound
    end
end

function LightExtension:setBeaconLightsVisibility(active, force, noEventSend)
	if self.lightExt.strobes ~= nil then
		if self.isSteerable then
			if self.beaconSync then
				self:setStrobeState(active)
			end
		else
			if self.attacherVehicle ~= nil then
				local rav = self:getRootAttacherVehicle()
				if rav.isSteerable then
					if rav.beaconSync == nil or rav.beaconSync then
						self:setStrobeState(active)
					end
				end
			end
		end
	end

	if self.lightExt.beaconSound ~= nil then
		if not self.lightExt.beaconSound.onStrobe then
			self:setBeaconSound(active)
		end
	end
end

function LightExtension:setLightsTypesMask(lightsTypesMask, force, noEventSend)
	if self.isSteerable and self.lightExt.extra ~= nil then
		if not self.lightExt.extra.inputOnly and self.extraLightEnabled then
			if bitAND(lightsTypesMask, 2^self.lightExt.extra.syncLightType) ~= 0 then
				self:setExtraLightState(self.lightControlActive)
			else
				self:setExtraLightState(false)
			end
        end
    end
end

function LightExtension:setUserLightState(active, noEventSend)
	if active ~= self.lightControlActive then
		if noEventSend == nil or noEventSend == false then
			if g_server ~= nil then
				g_server:broadcastEvent(LightExtExtraLightStateEvent:new(self, active), nil, nil, self)
			else
				g_client:getServerConnection():sendEvent(LightExtExtraLightStateEvent:new(self, active))
			end
		end

		self.lightControlActive = active

		self:setExtraLightState(active)
	end
end

function LightExtension:setExtraLightState(active)
	if self.lightExt.extra.deco ~= nil then
        setVisibility(self.lightExt.extra.deco, active)
    end

	if self.lightExt.extra.realLow ~= nil and self.lightExt.extra.realHigh ~= nil then
		local highProfile = self:getUseHighProfile()
		if highProfile then
			setVisibility(self.lightExt.extra.realHigh, active)
		else
			setVisibility(self.lightExt.extra.realLow, active)
		end
	end
end

function LightExtension:setStrobeState(active, force, noEventSend)
	if active ~= self.strobesActive or force then
		if noEventSend == nil or noEventSend == false then
			if g_server ~= nil then
				g_server:broadcastEvent(LightExtStrobeLightEvent:new(self, active), nil, nil, self)
			else
				g_client:getServerConnection():sendEvent(LightExtStrobeLightEvent:new(self, active))
			end
		end

		self.strobesActive = active

		-- If the attached trailer has strobs lets turn them on.
		for _,implement in pairs(self.attachedImplements) do
			if implement.object ~= nil then
				local connected = implement.object
				if connected.lightExt ~= nil and connected.lightExt.strobes ~= nil then
					connected:setStrobeState(active, true, true)
				-- else
					-- for _,connectedImplement in pairs(connected.attachedImplements) do
						-- if connectedImplement.object ~= nil then
							-- local connected = connectedImplement.object
							-- if connected.lightExt ~= nil and connected.lightExt.strobes ~= nil then
								-- connected:setStrobeState(active, true, true)
							-- end
						-- end
					-- end
				end
			end
		end

		if self.lightExt.beaconSound ~= nil then
			if self.lightExt.beaconSound.onStrobe then
				self:setBeaconSound(active)
			end
		end
	end
end

function LightExtension:onAttach(attacherVehicle, jointDescIndex)
	if self.lightExt.strobes ~= nil then
		if attacherVehicle.beaconSync ~= nil and not attacherVehicle.beaconSync then
			self:setStrobeState(attacherVehicle.strobesActive, true, true)
		end
	end
end

function LightExtension:onLeave()
	if self.lightExt.strobes ~= nil then
		if self:getDeactivateLights() then
			self:setStrobeState(false, true)
		end
	end
end

function LightExtension:onDetach()
	if self.lightExt.strobes ~= nil then
		self:setStrobeState(false, true)
	end
end

function LightExtension:onTurnedOff()
	if self.lightExt.cfw ~= nil then
		self.lightExt.cfw.counter = self.lightExt.cfw.seconds
		self.lightExt.cfw.a = true
	end
end

function LightExtension:mouseEvent(posX, posY, isDown, isUp, button)
end

function LightExtension:keyEvent(unicode, sym, modifier, isDown)
end


----------------------------------
-- LightExtStrobeLightEvent --
----------------------------------
LightExtStrobeLightEvent = {}
LightExtStrobeLightEvent_mt = Class(LightExtStrobeLightEvent, Event)
InitEventClass(LightExtStrobeLightEvent, "LightExtStrobeLightEvent")

function LightExtStrobeLightEvent:emptyNew()
    local self = Event:new(LightExtStrobeLightEvent_mt)
    return self
end;

function LightExtStrobeLightEvent:new(object, active)
    local self = LightExtStrobeLightEvent:emptyNew()
    self.object = object
    self.active = active
    return self
end;

function LightExtStrobeLightEvent:readStream(streamId, connection)
    self.object = readNetworkNodeObject(streamId)
    self.active = streamReadBool(streamId)
    self:run(connection)
end;

function LightExtStrobeLightEvent:writeStream(streamId, connection)
    writeNetworkNodeObject(streamId, self.object)
    streamWriteBool(streamId, self.active)
end;

function LightExtStrobeLightEvent:run(connection)
    self.object:setStrobeState(self.active, true, true)
    if not connection:getIsServer() then
        g_server:broadcastEvent(LightExtStrobeLightEvent:new(self.object, self.active), nil, connection, self.object)
    end
end


----------------------------------
-- LightExtExtraLightStateEvent --
----------------------------------
LightExtExtraLightStateEvent = {}
LightExtExtraLightStateEvent_mt = Class(LightExtExtraLightStateEvent, Event)
InitEventClass(LightExtExtraLightStateEvent, "LightExtExtraLightStateEvent")

function LightExtExtraLightStateEvent:emptyNew()
    local self = Event:new(LightExtExtraLightStateEvent_mt)
    return self
end;

function LightExtExtraLightStateEvent:new(object, active)
    local self = LightExtExtraLightStateEvent:emptyNew()
    self.object = object
    self.active = active
    return self
end;

function LightExtExtraLightStateEvent:readStream(streamId, connection)
    self.object = readNetworkNodeObject(streamId)
    self.active = streamReadBool(streamId)
    self:run(connection)
end;

function LightExtExtraLightStateEvent:writeStream(streamId, connection)
    writeNetworkNodeObject(streamId, self.object)
    streamWriteBool(streamId, self.active)
end;

function LightExtExtraLightStateEvent:run(connection)
    self.object:setUserLightState(self.active, true)
    if not connection:getIsServer() then
        g_server:broadcastEvent(LightExtExtraLightStateEvent:new(self.object, self.active), nil, connection, self.object)
    end
end