--[[
Interface: 1.5.1.0 b1580

Author: GtX | Andy
Date: 28.05.2017
Version: 1.0.0.0

History:
V 1.0.0.0 @ 28.05.2017 - Project Started
V 1.0.1.0 @ 27.04.2018 - Small error fix, added optional Real Lights.
V 1.0.2.0 @ 30.05.2018 - Add protection if rootAttacherVehicle can't be found.


NOTES:
Simple script to set the visibility of nodes on all connected trailers when the Root Vehicle is started.

Free to use and edit, Please keep the credits ;-)

EXAMPLE FOR XML:
<trailerRunLights decoration="0>5|0"/>
]]

TrailerRunLights = {};

function TrailerRunLights.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Attachable, specializations)
end

function TrailerRunLights:load(savegame)
	self.runLights = {}
	self.runLights.hasReal = false
	self.runLights.deco = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.trailerRunLights#decoration"))
	self.runLights.realLow = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.trailerRunLights#realLightLow"))
	self.runLights.realHigh = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.trailerRunLights#realLightHigh"))

	if self.runLights.deco ~= nil then
		setVisibility(self.runLights.deco, false)

		local hasLow = self.runLights.realLow ~= nil
		local hasHigh = self.runLights.realHigh ~= nil

		if hasLow and not hasHigh then
			print("Error: (TrailerRunLights.lua) high-profile light index missing in '"..self.configFileName.."'")
		elseif not hasLow and hasHigh then
			print("Error: (TrailerRunLights.lua) low-profile light index missing in '"..self.configFileName.."'")
		end

		if hasLow and hasHigh then
			setVisibility(self.runLights.realLow, false)
			setVisibility(self.runLights.realHigh, false)
			self.runLights.hasReal = true
		end
	else
		print("Error: (TrailerRunLights.lua) 'decoration' light missing in '"..self.configFileName.."'")
	end

	self.runLightsActive = false
end

function TrailerRunLights:delete()
end

function TrailerRunLights:update(dt)
	local rootAttacherVehicle = self:getRootAttacherVehicle()
	if self.attacherVehicle ~= nil and (rootAttacherVehicle ~= nil and rootAttacherVehicle.isMotorStarted) then
		if not self.runLightsActive then
			self.runLightsActive = true
			setVisibility(self.runLights.deco, true)
			if self.runLights.hasReal then
				local highProfile = self:getUseHighProfile()
				if highProfile then
					setVisibility(self.runLights.realLow, true)
				else
					setVisibility(self.runLights.realHigh, true)
				end
			end
		end
	else
		if self.runLightsActive then
			self.runLightsActive = false
			setVisibility(self.runLights.deco, false)
			if self.runLights.hasReal then
				if getVisibility(self.runLights.realLow) then
					setVisibility(self.runLights.realLow, false)
				end
				if getVisibility(self.runLights.realHigh) then
					setVisibility(self.runLights.realHigh, false)
				end
			end
		end
	end
end

function TrailerRunLights:updateTick(dt)
end

function TrailerRunLights:draw()
end

function TrailerRunLights:mouseEvent(posX, posY, isDown, isUp, button)
end

function TrailerRunLights:keyEvent(unicode, sym, modifier, isDown)
end

function TrailerRunLights:readStream(streamId, connection)
end

function TrailerRunLights:writeStream(streamId, connection)
end

function TrailerRunLights:readStreamUpdate(streamId, connection)
end

function TrailerRunLights:writeStreamUpdate(streamId, connection)
end